/*
 * sdl_display.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "display.hpp"
#include "my_exceptions.hpp"
#include "rstream/rstream_rwops.hpp"
#include "sdl_font.hpp"
#include "sdl_graphic.hpp"
#include "sdl_graphic_cache.hpp"
#include "sdl_pixels.hpp"

#include "boost/scoped_ptr.hpp"
#include <SDL.h>



//
// DisplayImpl struct for SDL
//

struct DisplayImpl {
	DisplayImpl();  // loads font, sets "screen" to SDL_GetVideoSurface() (for now)

	SDL_Surface *screen;
	SDL_Surface *font_surface;
	scoped_ptr<SDLFont> font;
};


//
// Display class for SDL
//

Display::Display()
	: pimpl(new DisplayImpl)
{
}

//
// begin & end drawing
//

void Display::begin()
{
	SDL_Surface *scr = SDL_GetVideoSurface();
	SDL_SetClipRect(scr, 0);
	SDL_FillRect(scr, 0, SDL_MapRGB(scr->format, 0, 0, 0));
}

void Display::end()
{
	SDL_Flip(SDL_GetVideoSurface());
}

//
// get width/height
//

int Display::getWidth() const
{
	return pimpl->screen->w;
}

int Display::getHeight() const
{
	return pimpl->screen->h;
}


//
// Clipping
//

void Display::setClipRectangle(int bl_x, int bl_y, int tr_x, int tr_y)
{
	SDL_Rect clip;
	clip.x = bl_x;
	clip.y = pimpl->screen->h - tr_y;  // need to give *top* for SDL
	clip.w = tr_x - bl_x;
	clip.h = tr_y - bl_y;
	SDL_SetClipRect(pimpl->screen, &clip);
}

void Display::clearClipRectangle()
{
	SDL_SetClipRect(pimpl->screen, 0);
}

//
// Font/Text Display
//

// The DisplayImpl ctor loads the font currently
DisplayImpl::DisplayImpl()
{
	font_surface = SDL_LoadBMP_RW(RWFromRStream("knights_sfont.bmp"), true);
	if (!font_surface) {
		string err("Error loading \"knights_sfont.bmp\". SDL error was: ");
		err += SDL_GetError();
		throw InitError(err);
	}

	font.reset(new SDLFont(font_surface));
	
	// also set "screen" to current display surface
	// (in future, might be possible to allow more than one display surface, eg separate windows?
	// that would require something more sophisticated than SdlApp2D though.)
	screen = SDL_GetVideoSurface();
}

void Display::drawText(int bl_x, int bl_y, const char *str)
{
	if (pimpl->font) {
		pimpl->font->drawText(pimpl->screen,
							  bl_x,
							  pimpl->screen->h - bl_y - pimpl->font->getTextHeight() + 1,
							  str,
							  255, 255, 255);
	}
}

int Display::getTextWidth(const char *str)
{
	if (pimpl->font) {
		return pimpl->font->getTextWidth(str);
	} else {
		return 0;
	}
}


//
// "Graphic" drawing
// This is just delegated to Graphic.
//

void Display::drawGraphic(int bl_x, int bl_y, const Graphic &gfx, bool darken)
{
	gfx.blit(pimpl->screen, bl_x, (pimpl->screen->h - 1) - bl_y - (gfx.getHeight() - 1), 
			 darken);
}

void Display::drawGraphicWithCC(int bl_x, int bl_y, const Graphic &gfx,
								const ColourChange &cc)
{
	const Graphic * newgfx = g_sdl_graphic_cache.getGraphicWithCC(gfx, cc);
	if (newgfx) {
		drawGraphic(bl_x, bl_y, *newgfx);
	}
}


//
// Pixel drawing
//

void Display::beginPixels()
{
	SDL_LockSurface(pimpl->screen);
}

void Display::drawPixel(int x, int y, int R, int G, int B)
{
	y = pimpl->screen->h - 1 - y;
	PlotPixel(pimpl->screen, x, y, R, G, B);
}

void Display::endPixels()
{
	SDL_UnlockSurface(pimpl->screen);
}
