/*
 * qconfig.hpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 * "Quick Configuration" library. This is intended as a simple, easy
 * to use store for global (constant) configuration values. The
 * variables are read from a text file (an istream) by the load
 * command, and can be accessed by the get commands. The text file is
 * a simple variable = value format, where the values can be integers,
 * floats, strings (enclosed in double quotes), or the words "true" or
 * "false".
 *
 * NOTE: This has basically been made obsolete by the new KConfig
 * library; however I'm keeping it around (for now) for backwards
 * compatibility.
 *
 */

#ifndef QCONFIG_HPP
#define QCONFIG_HPP

#include <iosfwd>
#include <string>
using namespace std;

class QConfig {
public:
	// Load configuration variables
	// (Can be called more than once if necessary)
	// This keeps reading variable/value pairs until it reaches EOF.
	static void Load(istream &str);

	// Access a variable
	// These throw if variable not found
	static int getInt(const string &name);
	static float getFloat(const string &name);
	static const string &getString(const string &name);
	static bool getBool(const string &name);

	// Forget all variables
	static void clear();

private:
	QConfig(); // not implemented
};


//
// exceptions
//

class QConfigError : public exception { 
public:
	virtual ~QConfigError() throw() { }
};

// Thrown if there is a problem during load()
class QConfigLoadError : public QConfigError {
public:
	explicit QConfigLoadError(const string &n_) : n(n_) { }
	virtual ~QConfigLoadError() throw() { }
	virtual const char *what() const throw();
private:
	string n;
};

// Thrown by getXXX() if a variable cannot be cast to the requested type
// NB converting float to int isn't allowed. (other way round is ok of course)
class QConfigTypeMismatch : public QConfigError {
public:
	explicit QConfigTypeMismatch(const string &n_) : n(n_) { }
	virtual ~QConfigTypeMismatch() throw() { }
	virtual const char *what() const throw();
private:
	string n;
};

// Thrown by getXXX() if a variable does not exist
class QConfigNotFound : public QConfigError {
public:
	explicit QConfigNotFound(const string &n_) : n(n_) { }
	virtual ~QConfigNotFound() throw() { }
	virtual const char *what() const throw();
private:
	string n;
};


#endif
